"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var matrix_1 = require("./matrix");
var rectangle_1 = require("./rectangle");
var utils_1 = require("./utils");
function getByCirculatedIndex(ary, index) {
    var found = ary[(ary.length + index) % ary.length];
    if (found === undefined) {
        throw new Error('Can not find an element');
    }
    return found;
}
;
function clearTopSide(matrix, width) {
    return matrix.map(function (row, y) {
        if (y < width) {
            return row.map(function (element) { return Object.assign({}, element, { symbol: null }); });
        }
        return row;
    });
}
exports.clearTopSide = clearTopSide;
;
function clearBottomSide(matrix, width) {
    var maxY = matrix_1.getMaxY(matrix);
    return matrix.map(function (row, y) {
        if (y > maxY - width) {
            return row.map(function (element) { return Object.assign({}, element, { symbol: null }); });
        }
        return row;
    });
}
exports.clearBottomSide = clearBottomSide;
;
function clearLeftSide(matrix, width) {
    return matrix.map(function (row) {
        var newRow = row.slice();
        for (var x = 0; x < width; x += 1) {
            newRow[x] = Object.assign({}, newRow[x], { symbol: null });
        }
        return newRow;
    });
}
exports.clearLeftSide = clearLeftSide;
;
function clearRightSide(matrix, width) {
    var maxX = matrix_1.getMaxX(matrix);
    return matrix.map(function (row) {
        var newRow = row.slice();
        for (var x = maxX; x > maxX - width; x -= 1) {
            newRow[x] = Object.assign({}, newRow[x], { symbol: null });
        }
        return newRow;
    });
}
exports.clearRightSide = clearRightSide;
;
function drawTopSide(matrix, borderWidth, symbols, fromX, toX) {
    return matrix.map(function (row, y) {
        if (y < borderWidth) {
            return row.map(function (element, x) {
                if (x >= fromX && x <= toX && symbols.length > 0) {
                    return Object.assign({}, element, {
                        symbol: getByCirculatedIndex(symbols, y),
                    });
                }
                return element;
            });
        }
        return row;
    });
}
exports.drawTopSide = drawTopSide;
;
function drawBottomSide(matrix, borderWidth, symbols, fromX, toX) {
    var maxY = matrix_1.getMaxY(matrix);
    return matrix.map(function (row, y) {
        if (y > maxY - borderWidth) {
            return row.map(function (element, x) {
                if (x >= fromX && x <= toX && symbols.length > 0) {
                    return Object.assign({}, element, {
                        symbol: getByCirculatedIndex(symbols, maxY - y),
                    });
                }
                return element;
            });
        }
        return row;
    });
}
exports.drawBottomSide = drawBottomSide;
;
function drawLeftSide(matrix, borderWidth, symbols, fromY, toY) {
    return matrix.map(function (row, y) {
        if (y >= fromY && y <= toY) {
            return row.map(function (element, x) {
                if (x < borderWidth && symbols.length > 0) {
                    return Object.assign({}, element, {
                        symbol: getByCirculatedIndex(symbols, x),
                    });
                }
                return element;
            });
        }
        return row;
    });
}
exports.drawLeftSide = drawLeftSide;
;
function drawRightSide(matrix, borderWidth, symbols, fromY, toY) {
    var maxX = matrix_1.getMaxX(matrix);
    return matrix.map(function (row, y) {
        if (y >= fromY && y <= toY) {
            return row.map(function (element, x) {
                if (x > maxX - borderWidth && symbols.length > 0) {
                    return Object.assign({}, element, {
                        symbol: getByCirculatedIndex(symbols, maxX - x),
                    });
                }
                return element;
            });
        }
        return row;
    });
}
exports.drawRightSide = drawRightSide;
;
function drawCorner(matrix, rectangle, symbols) {
    var counter = 0;
    return matrix.map(function (row, y) {
        if (y >= rectangle.y && y < rectangle.y + rectangle.height) {
            return row.map(function (element, x) {
                if (x >= rectangle.x && x < rectangle.x + rectangle.width) {
                    var newRow = Object.assign({}, element, {
                        symbol: getByCirculatedIndex(symbols, counter),
                    });
                    counter += 1;
                    return newRow;
                }
                return element;
            });
        }
        return row;
    });
}
exports.drawCorner = drawCorner;
;
function shrinkContentAreaByBorders(matrix, borders) {
    return rectangle_1.shrinkRectangle(matrix_1.matrixToRectangle(matrix), {
        top: borders.topWidth,
        bottom: borders.bottomWidth,
        left: borders.leftWidth,
        right: borders.rightWidth,
    });
}
function validateMatrixWithBorders(matrix, borders) {
    var contentArea = shrinkContentAreaByBorders(matrix, borders);
    return utils_1.validateSize(rectangle_1.rectangleToSize(contentArea));
}
exports.validateMatrixWithBorders = validateMatrixWithBorders;
function placeBorders(matrix, borders) {
    if (validateMatrixWithBorders(matrix, borders) === false) {
        throw new Error('Borders do not fit in the matrix');
    }
    var maxWidth = matrix_1.getWidth(matrix);
    var maxHeight = matrix_1.getHeight(matrix);
    var newMatrix = matrix;
    newMatrix = clearTopSide(newMatrix, borders.topWidth);
    newMatrix = clearBottomSide(newMatrix, borders.bottomWidth);
    newMatrix = clearLeftSide(newMatrix, borders.leftWidth);
    newMatrix = clearRightSide(newMatrix, borders.rightWidth);
    newMatrix = drawTopSide(newMatrix, borders.topWidth, borders.topSymbols, borders.leftWidth, maxWidth - borders.rightWidth);
    newMatrix = drawBottomSide(newMatrix, borders.bottomWidth, borders.bottomSymbols, borders.leftWidth, maxWidth - borders.rightWidth);
    newMatrix = drawLeftSide(newMatrix, borders.leftWidth, borders.leftSymbols, borders.topWidth, maxHeight - borders.bottomWidth);
    newMatrix = drawRightSide(newMatrix, borders.rightWidth, borders.rightSymbols, borders.topWidth, maxHeight - borders.bottomWidth);
    newMatrix = drawCorner(// top-left
    newMatrix, { x: 0, y: 0,
        width: borders.leftWidth, height: borders.topWidth }, borders.topLeftSymbols);
    newMatrix = drawCorner(// top-right
    newMatrix, { x: maxWidth - borders.rightWidth, y: 0,
        width: borders.rightWidth, height: borders.topWidth }, borders.topRightSymbols);
    newMatrix = drawCorner(// bottom-left
    newMatrix, { x: 0, y: maxHeight - borders.bottomWidth,
        width: borders.leftWidth, height: borders.bottomWidth }, borders.bottomLeftSymbols);
    newMatrix = drawCorner(// bottom-right
    newMatrix, { x: maxWidth - borders.rightWidth, y: maxHeight - borders.bottomWidth,
        width: borders.rightWidth, height: borders.bottomWidth }, borders.bottomRightSymbols);
    return {
        matrix: newMatrix,
        contentArea: shrinkContentAreaByBorders(matrix, borders),
    };
}
exports.placeBorders = placeBorders;
