import { Coordinate, Size } from './utils';
import { Rectangle } from './rectangle';
export declare type ElementSymbol = string;
export declare type SymbolRuler = (symbol: ElementSymbol) => 0 | 1 | 2;
export declare type ElementStyle = {
    foregroundColor: string;
    backgroundColor: string;
    bold: boolean;
    dim: boolean;
    italic: boolean;
    underline: boolean;
    inverse: boolean;
    hidden: boolean;
    strikethrough: boolean;
};
export declare type ElementBody = {
    symbol: ElementSymbol | null;
    style: ElementStyle;
};
export declare type Element = {
    x: number;
    y: number;
    symbol: ElementSymbol | null | false;
    style: ElementStyle;
};
export declare type Matrix = Element[][];
export declare type PourableElement = {
    isLineBreaking: boolean;
    symbol: ElementSymbol;
    symbolWidth: number;
    style: ElementStyle;
};
export declare type PouredElement = {
    isLineBreaking: PourableElement['isLineBreaking'];
    symbol: PourableElement['symbol'];
    symbolWidth: PourableElement['symbolWidth'];
    style: PourableElement['style'];
    x: number;
    y: number;
};
export declare function createDefaultElementStyle(): ElementStyle;
export declare function createElementBody(symbol: ElementBody['symbol']): ElementBody;
export declare function createMatrix(size: Size, defaultSymbol?: ElementSymbol | null): Matrix;
/**
 * Validate that the matrix is not empty and is rectangular
 */
export declare function validateMatrix(matrix: Matrix): boolean;
export declare function getElement(matrix: Matrix, coordinate: Coordinate): Element | null;
/**
 * A function that makes it easy to create a matrix from text mainly for testing.
 *
 * @param text Text consisting only of ASCII characters.
 *             It also needs to be rectangular.
 */
export declare function createMatrixFromText(text: string): Matrix;
export declare function getWidth(matrix: Matrix): number;
export declare function getHeight(matrix: Matrix): number;
export declare function getMaxX(matrix: Matrix): number;
export declare function getMaxY(matrix: Matrix): number;
export declare function matrixToRectangle(matrix: Matrix): Rectangle;
export declare function overwriteMatrix(matrix: Matrix, replacer: Matrix, coordinate: Coordinate, symbolRuler: SymbolRuler): Matrix;
export declare function cropMatrix(matrix: Matrix, rectangle: Rectangle): Matrix;
/**
 * @param character A single character with or without ANSI escape codes.
 *                  It is mainly assumed to pass the return value of "slice-ansi-string".
 */
export declare function decodeAnsiStyles(character: string): Partial<ElementStyle>;
export declare function parseContent(content: string, symbolRuler: SymbolRuler): PourableElement[];
/**
 * Try to pour elements to a certain width
 */
export declare function pourElementsVirtually(pourableElements: PourableElement[], matrixWidth: number): {
    pouredElements: PouredElement[];
    contentHeight: number;
};
export declare function pourContent(matrix: Matrix, pourableElements: PourableElement[], startingY: number): Matrix;
export declare function renderMatrix(matrix: Matrix, backgroundSymbol: ElementSymbol): string;
